@@ -68,8 +68,6 @@ public:
 	void ModeChange(const CString& sModes, const CString& sNick = "");
 	bool AddMode(unsigned char uMode, const CString& sArg);
 	bool RemMode(unsigned char uMode);
-	void OnOp(const CString& sOpNick, const CString& sNick, bool bOpped);
-	void OnVoice(const CString& sOpNick, const CString& sNick, bool bVoiced);
 	CString GetModeString() const;
 	CString GetModeArg(CString& sArgs) const;
 	CString GetModeForNames() const;
@@ -104,7 +102,7 @@ public:
 	void SetTopicOwner(const CString& s) { m_sTopicOwner = s; }
 	void SetTopicDate(unsigned long u) { m_ulTopicDate = u; }
 	void SetDefaultModes(const CString& s) { m_sDefaultModes = s; }
-	void SetBufferCount(unsigned int u) { m_uBufferCount = u; }
+	bool SetBufferCount(unsigned int u, bool bForce = false);
 	void SetKeepBuffer(bool b) { m_bKeepBuffer = b; }
 	void SetDetached(bool b = true) { m_bDetached = b; }
 	void SetInConfig(bool b) { m_bInConfig = b; }
