@@ -78,6 +78,7 @@ public:
 		m_bGotPass = false;
 		m_bGotNick = false;
 		m_bGotUser = false;
+		m_bInCap = false;
 		m_bNamesx = false;
 		m_bUHNames = false;
 		EnableReadLine();
@@ -112,6 +113,8 @@ public:
 	void PutModule(const CString& sModule, const CString& sLine);
 	void PutModNotice(const CString& sModule, const CString& sLine);
 
+	bool IsCapEnabled(const CString& sCap) { return 1 == m_ssAcceptedCaps.count(sCap); }
+
 	virtual void ReadLine(const CString& sData);
 	bool SendMotd();
 	void HelpUser();
@@ -127,11 +130,14 @@ public:
 	const CIRCSock* GetIRCSock() const;
 	CIRCSock* GetIRCSock();
 private:
+	void HandleCap(const CString& sLine);
+	void RespondCap(const CString& sResponse);
 
 protected:
 	bool                 m_bGotPass;
 	bool                 m_bGotNick;
 	bool                 m_bGotUser;
+	bool                 m_bInCap;
 	bool                 m_bNamesx;
 	bool                 m_bUHNames;
 	CUser*               m_pUser;
@@ -139,6 +145,7 @@ protected:
 	CString              m_sPass;
 	CString              m_sUser;
 	CSmartPtr<CAuthBase> m_spAuth;
+	SCString             m_ssAcceptedCaps;
 };
 
 #endif // !_CLIENT_H
