@@ -40,6 +40,7 @@ public:
 	bool OnChanMsg(CNick& Nick, const CString& sChan, CString& sMessage);
 	bool OnPrivNotice(CNick& Nick, CString& sMessage);
 	bool OnChanNotice(CNick& Nick, const CString& sChan, CString& sMessage);
+	bool OnServerCapAvailable(const CString& sCap);
 	// !Message Handlers
 
 	virtual void ReadLine(const CString& sData);
@@ -54,6 +55,16 @@ public:
 	void ResetChans();
 	void Quit(const CString& sQuitMsg = "");
 
+	/** You can call this from CModule::OnServerCapResult to suspend
+	 *  sending other CAP requests and CAP END for a while. Each
+	 *  call to PauseCap should be balanced with a call to ResumeCap.
+	 */
+	void PauseCap();
+	/** If you used PauseCap, call this when CAP negotiation and logging in
+	 *  should be resumed again.
+	 */
+	void ResumeCap();
+
 	// Setters
 	void SetPass(const CString& s) { m_sPass = s; }
 	// !Setters
@@ -76,6 +87,7 @@ public:
 	const set<unsigned char>& GetUserModes() const { return m_scUserModes; }
 	// This is true if we are past raw 001
 	bool IsAuthed() const { return m_bAuthed; }
+	bool IsCapAccepted(const CString& sCap) { return 1 == m_ssAcceptedCaps.count(sCap); }
 	// !Getters
 
 	// This handles NAMESX and UHNAMES in a raw 353 reply
@@ -86,6 +98,7 @@ private:
 	void ParseISupport(const CString& sLine);
 	// This is called when we connect and the nick we want is already taken
 	void SendAltNick(const CString& sBadNick);
+	void SendNextCap();
 protected:
 	bool                                m_bISpoofReleased;
 	bool                                m_bAuthed;
@@ -100,6 +113,9 @@ protected:
 	CString                             m_sPass;
 	map<CString, CChan*>                m_msChans;
 	unsigned int                        m_uMaxNickLen;
+	unsigned int                        m_uCapPaused;
+	SCString                            m_ssAcceptedCaps;
+	SCString                            m_ssPendingCaps;
 };
 
 #endif // !_IRCSOCK_H
