@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -20,14 +20,14 @@ public:
 	unsigned short GetPort() const;
 	const CString& GetPass() const;
 	bool IsSSL() const;
-	CString GetString() const;
+	CString GetString(bool bIncludePassword = true) const;
 	static bool IsValidHostName(const CString& sHostName);
 private:
 protected:
-	CString			m_sName;
-	unsigned short	m_uPort;
-	CString			m_sPass;
-	bool			m_bSSL;
+	CString         m_sName;
+	unsigned short  m_uPort;
+	CString         m_sPass;
+	bool            m_bSSL;
 };
 
 #endif // !_SERVER_H
