@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -26,11 +26,22 @@ using std::pair;
 using std::cout;
 using std::endl;
 
+/** Output a debug info if debugging is enabled.
+ *  If ZNC was compiled with <code>--enable-debug</code> or was started with
+ *  <code>--debug</code>, the given argument will be sent to stdout.
+ *
+ *  You can use all the features of C++ streams:
+ *  @code
+ *  DEBUG("I had " << errors << " errors");
+ *  @endcode
+ *
+ *  @param f The expression you want to display.
+ */
 #define DEBUG(f) do { \
 	if (CUtils::Debug()) { \
 		cout << f << endl; \
 	} \
-} while (0);
+} while (0)
 
 static inline void SetFdCloseOnExec(int fd)
 {
@@ -51,6 +62,7 @@ public:
 	static CString GetIP(unsigned long addr);
 	static unsigned long GetLongIP(const CString& sIP);
 	static void SetStdoutIsTTY(bool b) { stdoutIsTTY = b; }
+	static bool StdoutIsTTY() { return stdoutIsTTY; }
 	static void SetDebug(bool b) { debug = b; }
 	static bool Debug() { return debug; }
 
@@ -59,10 +71,13 @@ public:
 	static void PrintPrompt(const CString& sMessage);
 	static void PrintAction(const CString& sMessage);
 	static void PrintStatus(bool bSuccess, const CString& sMessage = "");
-	static CString GetHashPass();
+
+	static const CString sDefaultHash;
+
 	static CString GetSaltedHashPass(CString& sSalt);
 	static CString GetSalt();
-	static CString SaltedHash(const CString& sPass, const CString& sSalt);
+	static CString SaltedMD5Hash(const CString& sPass, const CString& sSalt);
+	static CString SaltedSHA256Hash(const CString& sPass, const CString& sSalt);
 	static CString GetPass(const CString& sPrompt);
 	static bool GetInput(const CString& sPrompt, CString& sRet, const CString& sDefault = "", const CString& sHint = "");
 	static bool GetBoolInput(const CString& sPrompt, bool bDefault);
@@ -102,30 +117,89 @@ public:
 	EType GetType() const { return m_eType; }
 private:
 protected:
-	EType	m_eType;
+	EType  m_eType;
 };
 
 
+/** Generate a grid-like output from a given input.
+ *
+ *  @code
+ *  CTable table;
+ *  table.AddColumn("a");
+ *  table.AddColumn("b");
+ *  table.AddRow();
+ *  table.SetCell("a", "hello");
+ *  table.SetCell("b", "world");
+ *
+ *  unsigned int idx = 0;
+ *  CString tmp;
+ *  while (table.GetLine(idx++, tmp)) {
+ *      // Output tmp somehow
+ *  }
+ *  @endcode
+ *
+ *  The above code would generate the following output:
+ *  @verbatim
++-------+-------+
+| a     | b     |
++-------+-------+
+| hello | world |
++-------+-------+@endverbatim
+ */
 class CTable : protected vector<vector<CString> > {
 public:
 	CTable() {}
 	virtual ~CTable() {}
 
+	/** Adds a new column to the table.
+	 *  Please note that you should add all columns before starting to fill
+	 *  the table!
+	 *  @param sName The name of the column.
+	 *  @return false if a column by that name already existed.
+	 */
 	bool AddColumn(const CString& sName);
+
+	/** Adds a new row to the table.
+	 *  After calling this you can fill the row with content.
+	 *  @return The index of this row
+	 */
 	unsigned int AddRow();
+
+	/** Sets a given cell in the table to a value.
+	 *  @param sColumn The name of the column you want to fill.
+	 *  @param sValue The value to write into that column.
+	 *  @param uRowIdx The index of the row to use as returned by AddRow().
+	 *                 If this is not given, the last row will be used.
+	 *  @return True if setting the cell was successful.
+	 */
 	bool SetCell(const CString& sColumn, const CString& sValue, unsigned int uRowIdx = ~0);
+
+	/** Get a line of the table's output
+	 *  @param uIdx The index of the line you want.
+	 *  @param sLine This string will receive the output.
+	 *  @return True unless uIdx is past the end of the table.
+	 */
 	bool GetLine(unsigned int uIdx, CString& sLine) const;
 
+	/** Return the width of the given column.
+	 *  Please note that adding and filling new rows might change the
+	 *  result of this function!
+	 *  @param uIdx The index of the column you are interested in.
+	 *  @return The width of the column.
+	 */
 	unsigned int GetColumnWidth(unsigned int uIdx) const;
 
+	/// Completely clear the table.
 	void Clear();
+
+	/// @return The number of rows in this table, not counting the header.
 	using vector<vector<CString> >::size;
 private:
 	unsigned int GetColumnIndex(const CString& sName) const;
 
 protected:
-	vector<CString>				m_vsHeaders;
-	map<CString, unsigned int>	m_msuWidths;	// Used to cache the width of a column
+	vector<CString>            m_vsHeaders;
+	map<CString, unsigned int> m_msuWidths;  // Used to cache the width of a column
 };
 
 
@@ -157,9 +231,9 @@ public:
 	CString Crypt(const CString & sData);
 
 private:
-	unsigned char		*m_ivec;
-	BF_KEY 				m_bkey;
-	int					m_iEncrypt, m_num;
+	unsigned char  *m_ivec;
+	BF_KEY          m_bkey;
+	int             m_iEncrypt, m_num;
 };
 
 #endif /* HAVE_LIBSSL */
@@ -192,13 +266,12 @@ public:
 	 * @param uTTL the time-to-live for this specific item
 	 */
 	void AddItem(const K& Item, unsigned int uTTL) {
-		if (!uTTL) {			// If time-to-live is zero we don't want to waste our time adding it
-			RemItem(Item);		// Remove the item incase it already exists
+		if (!uTTL) {            // If time-to-live is zero we don't want to waste our time adding it
+			RemItem(Item);  // Remove the item incase it already exists
 			return;
 		}
 
 		m_mItems[Item] = value(CUtils::GetMillTime() + uTTL, V());
-		Cleanup();
 	}
 
 	/**
@@ -217,13 +290,12 @@ public:
 	 * @param uTTL the time-to-live for this specific item
 	 */
 	void AddItem(const K& Item, const V& Val, unsigned int uTTL) {
-		if (!uTTL) {			// If time-to-live is zero we don't want to waste our time adding it
-			RemItem(Item);		// Remove the item incase it already exists
+		if (!uTTL) {             // If time-to-live is zero we don't want to waste our time adding it
+			RemItem(Item);   // Remove the item incase it already exists
 			return;
 		}
 
 		m_mItems[Item] = value(CUtils::GetMillTime() + uTTL, Val);
-		Cleanup();
 	}
 
 	/**
@@ -273,57 +345,24 @@ public:
 		}
 	}
 
+	/**
+	 * @brief Clear all entries
+	 */
+	void Clear() {
+		m_mItems.clear();
+	}
+
 	// Setters
 	void SetTTL(unsigned int u) { m_uTTL = u; }
 	// !Setters
 	// Getters
-	unsigned int GetTTL() { return m_uTTL; }
+	unsigned int GetTTL() const { return m_uTTL; }
 	// !Getters
-private:
+protected:
 	typedef pair<unsigned long long, V> value;
 	typedef typename map<K, value>::iterator iterator;
-	map<K, value>	m_mItems;	//!< Map of cached items.  The value portion of the map is for the expire time
-	unsigned int	m_uTTL;					//!< Default time-to-live duration
-};
-
-/**
- * @class CNoCopy
- * @author prozac <prozac@rottenboy.com>
- * @brief This class is intended to be derived from to prevent copying of your derived class, the implementations of the copy constructor and equals operator were intentionally made private and omitted
- */
-class CNoCopy {
-protected:
-	CNoCopy() {}	// Allow construction
-	~CNoCopy() {}	// and destruction of derived objects
-private:
-	CNoCopy(const CNoCopy&);				// Disallow copying
-	CNoCopy& operator =(const CNoCopy&);	// and assignment
-};
-
-/**
- * @class CSafePtr
- * @author prozac <prozac@rottenboy.com>
- * @brief Safe deletion of a pointer.
- *
- * This class is intended to be created on the stack and hold a pointer which
- * will be deleted upon destruction.  It is useful for functions where you need
- * an allocated pointer and have many return paths.  It prevents copying to get
- * around the exclusive ownership situation which makes std::auto_ptr
- * invalidate the first pointer on copy.  This is intended to be used in
- * simplistic situations such as local variables.
- */
-template<typename T>
-class CSafePtr : private CNoCopy {
-public:
-	CSafePtr() { m_pType = new T; }
-	CSafePtr(T* p) { m_pType = p; }
-	virtual ~CSafePtr() { delete m_pType; }
-
-	T& operator *() const { assert(m_pType); return *m_pType; }
-	T* operator ->() const { assert(m_pType); return m_pType; }
-private:
-	operator T() const;
-	T*	m_pType;
+	map<K, value>   m_mItems;   //!< Map of cached items.  The value portion of the map is for the expire time
+	unsigned int    m_uTTL;     //!< Default time-to-live duration
 };
 
 /**
@@ -374,8 +413,6 @@ public:
 	// Overloaded operators
 	T& operator *() const { assert(m_pType); return *m_pType; }
 	T* operator ->() const { assert(m_pType); return m_pType; }
-	bool operator ==(T* rhs) { return (m_pType == rhs); }
-	bool operator ==(const CSmartPtr<T>& rhs) { return (m_pType == *rhs); }
 
 	/**
 	 * @brief Attach() to a raw pointer
@@ -390,20 +427,18 @@ public:
 	 * @return Reference to self
 	 */
 	CSmartPtr<T>& operator =(const CSmartPtr<T>& CopyFrom) {
-		if (&CopyFrom != this) {				// Check for assignment to self
-			Release();							// Release the current pointer
+		if (&CopyFrom != this) {              // Check for assignment to self
+			Release();                    // Release the current pointer
 
-			if (CopyFrom.IsNull()) {			// If the source raw pointer is null
-				return *this;					// Then just bail out
+			if (CopyFrom.IsNull()) {      // If the source raw pointer is null
+				return *this;         // Then just bail out
 			}
 
-			m_pType = &(*CopyFrom);				// Make our pointers reference the same raw pointer and counter
+			m_pType = CopyFrom.m_pType;   // Make our pointers reference the same raw pointer and counter
 			m_puCount = CopyFrom.m_puCount;
 
-			if (m_pType) {						// If we now point to something valid, increment the counter
-				assert(m_puCount);
-				(*m_puCount)++;
-			}
+			assert(m_puCount);            // We now point to something valid, so increment the counter
+			(*m_puCount)++;
 		}
 
 		return *this;
@@ -432,12 +467,12 @@ public:
 	 * @return Reference to self
 	 */
 	CSmartPtr<T>& Attach(T* pRawPtr) {
-		if (pRawPtr != m_pType) {					// Check for assignment to self
-			Release();								// Release the current pointer
-			m_pType = pRawPtr;						// Point to the passed raw pointer
+		if (pRawPtr != m_pType) {                        // Check for assignment to self
+			Release();                               // Release the current pointer
+			m_pType = pRawPtr;                       // Point to the passed raw pointer
 
-			if (m_pType) {							// If the passed pointer was valid
-				m_puCount = new unsigned int(1);	// Create a new counter starting at 1 (us)
+			if (m_pType) {                           // If the passed pointer was valid
+				m_puCount = new unsigned int(1); // Create a new counter starting at 1 (us)
 			}
 		}
 
@@ -448,29 +483,37 @@ public:
 	 * @brief Releases the underlying raw pointer and cleans up if we were the last reference to said pointer
 	 */
 	void Release() {
-		if (m_pType) {				// Only release if there is something to be released
+		if (m_pType) {              // Only release if there is something to be released
 			assert(m_puCount);
-			(*m_puCount)--;			// Decrement our counter
+			(*m_puCount)--;     // Decrement our counter
 
-			if (!*m_puCount) {		// If we were the last reference to this pointer, then clean up
+			if (!*m_puCount) {  // If we were the last reference to this pointer, then clean up
 				delete m_puCount;
 				delete m_pType;
 			}
 
-			m_pType = NULL;			// Get rid of our references
+			m_pType = NULL;     // Get rid of our references
 			m_puCount = NULL;
 		}
 	}
 
 	// Getters
 	T* GetPtr() const { return m_pType; }
-	const unsigned int* GetCount() const { return m_puCount; }
-	unsigned int GetClientCount() const { return (m_puCount) ? *m_puCount : 0; }
+	unsigned int GetCount() const { return (m_puCount) ? *m_puCount : 0; }
 	// !Getters
 private:
-	T*				m_pType;	//!< Raw pointer to the class being referenced
-	unsigned int*	m_puCount;	//!< Counter of how many CSmartPtr's are referencing the same raw pointer
+	T*            m_pType;      //!< Raw pointer to the class being referenced
+	unsigned int* m_puCount;    //!< Counter of how many CSmartPtr's are referencing the same raw pointer
 };
 
+template<typename T>
+bool operator ==(T* lhs, const CSmartPtr<T>& rhs) { return (lhs == rhs.GetPtr()); }
+
+template<typename T>
+bool operator ==(const CSmartPtr<T>& lhs, T* rhs) { return (lhs.GetPtr() == rhs); }
+
+template<typename T>
+bool operator ==(const CSmartPtr<T>& lhs, const CSmartPtr<T>& rhs) { return (lhs.GetPtr() == rhs.GetPtr()); }
+
 #endif // !_UTILS_H
 
