@@ -38,7 +38,7 @@ public:
 	void ReleaseISpoof();
 	bool WritePidFile(int iPid);
 	bool DeletePidFile();
-	CUser* GetUser(const CString& sUser);
+	bool WaitForChildLock();
 	Csock* FindSockByName(const CString& sSockName);
 	bool IsHostAllowed(const CString& sHostMask) const;
 	// This returns false if there are too many anonymous connections from this ip
@@ -53,9 +53,9 @@ public:
 	static CString GetVersion();
 	static CString GetTag(bool bIncludeVersion = true);
 	CString GetUptime() const;
-	void ClearVHosts();
-	bool AddVHost(const CString& sHost);
-	bool RemVHost(const CString& sHost);
+	void ClearBindHosts();
+	bool AddBindHost(const CString& sHost);
+	bool RemBindHost(const CString& sHost);
 	void Broadcast(const CString& sMessage, bool bAdminOnly = false,
 			CUser* pSkipUser = NULL, CClient* pSkipClient = NULL);
 	void AddBytesRead(unsigned long long u) { m_uBytesRead += u; }
@@ -84,6 +84,7 @@ public:
 	void SetStatusPrefix(const CString& s) { m_sStatusPrefix = (s.empty()) ? "*" : s; }
 	void SetISpoofFile(const CString& s) { m_sISpoofFile = s; }
 	void SetISpoofFormat(const CString& s) { m_sISpoofFormat = (s.empty()) ? "global { reply \"%\" }" : s; }
+	void SetMaxBufferSize(unsigned int i) { m_uiMaxBufferSize = i; }
 	// !Setters
 
 	// Getters
@@ -100,14 +101,15 @@ public:
 	CString GetConfPath(bool bAllowMkDir = true) const;
 	CString GetUserPath() const;
 	CString GetModPath() const;
-	CString GetPemLocation() const { return GetZNCPath() + "/znc.pem"; }
+	CString GetPemLocation() const { return m_sSSLCertFile; }
 	const CString& GetConfigFile() const { return m_sConfigFile; }
 	bool WritePemFile();
 	const CString& GetISpoofFile() const { return m_sISpoofFile; }
 	const CString& GetISpoofFormat() const { return m_sISpoofFormat; }
-	const VCString& GetVHosts() const { return m_vsVHosts; }
+	const VCString& GetBindHosts() const { return m_vsBindHosts; }
 	const vector<CListener*>& GetListeners() const { return m_vpListeners; }
 	time_t TimeStarted() const { return m_TimeStarted; }
+	unsigned int GetMaxBufferSize() const { return m_uiMaxBufferSize; }
 	// !Getters
 
 	// Static allocator
@@ -166,12 +168,14 @@ protected:
 	CString                m_sOrigISpoof;
 	CString                m_sISpoofFormat;
 	CString                m_sPidFile;
-	VCString               m_vsVHosts;
+	CString                m_sSSLCertFile;
+	VCString               m_vsBindHosts;
 	VCString               m_vsMotd;
 	CFile                  m_LockFile;
 	CFile*                 m_pISpoofLockFile;
 	unsigned int           m_uiConnectDelay;
 	unsigned int           m_uiAnonIPLimit;
+	unsigned int           m_uiMaxBufferSize;
 	CGlobalModules*        m_pModules;
 	unsigned long long     m_uBytesRead;
 	unsigned long long     m_uBytesWritten;
